/*
 *  ffcounter.js - A simple "odometer-style" counter for Firefox downloads.
 *
 *  History:
 *      v0.1 - Initial version. Untested in IE. Inefficient handling of updates.
 *
 *  Copyright 2005 Infocraft
 *  http://www.infocraft.com/
 *
 *  The latest version of this script can be found at:
 *  http://www.infocraft.com/projects/ffcounter/ffcounter.js
 *
 *  You may use this script in its entirety, modify it to suit your needs,
 *  adapt its functions, or just use it for inspiration.  However, some form
 *  of acknowledgement, especially a link back to http://www.infocraft.com/, is
 *  always appreciated.
 */

/***************************  Adjustable Parameters ***************************/

var counter_url = "/fxcounter/fxcounter.php";	// The URL to your counter mirror.
var counter_id = "fxcounter";	// The id of the counter in your HTML.
var update_interval = 250;		// The interval, in ms, to update the counter.
var resync_interval = 60000;	// The interval, in ms, to resync with the feed.

/***************************     Static Globals     ***************************/

var current_count = null;
var initial_count = null;
var initial_time = null;
var download_rate = null;
var request = null;
var counter_on = false;

/***************************    Public Functions    ***************************/

// This should be called from your HTML, probably as an onload="get_count();"
// event in your <body> tag.
function get_count()
{
	var delay = (!initial_time || !initial_count) ? 10000 : resync_interval;

	load_xml(counter_url);
	setTimeout("get_count()", delay);
}

/***************************    Private Functions   ***************************/


// Adapted from the good people at ADC:
// http://developer.apple.com/internet/webcontent/xmlhttpreq.html
// this starts loading the XML data and sets up the process_request handler.
function load_xml(url) 
{
	// branch for native XMLHttpRequest object
	if (window.XMLHttpRequest) {
		request = new XMLHttpRequest();
		request.onreadystatechange = process_request;
		request.open("GET", url, true);
		request.send(null);

	// branch for IE/Windows ActiveX version
	} else if (window.ActiveXObject) {
		request = new ActiveXObject("Microsoft.XMLHTTP");
		if (request) {
			request.onreadystatechange = process_request;
			request.open("GET", url, true);
			request.send();
		}
	}
}

// This function is called once the request state changes.  It pulls the
// relevant data from the response and sends to update_count().
function process_request()
{
	var response;
	var time;
	var count_item;
	var count;

	if (request.readyState == 4 && request.status == 200) {
		response = request.responseXML;
		time = response.getElementsByTagName('pubDate')[0].firstChild.data;
		count_item = response.getElementsByTagName('item')[0];
		count = count_item.getElementsByTagName('description')[0].firstChild.data;
		update_count(time, count);
	}
}

// Now that the data from the XML request has been extracted, this updates the
// global variables with that data.
function update_count(time, count)
{

	if (!initial_count) {
		initial_time = new Date(time);
		initial_count = count.replace(/\D/g,'');
	} else {
		var last_time = new Date(time);
		var last_count = count.replace(/\D/g,'');
		download_rate = calculate_rate(last_time, last_count);
	}

	current_count = parseFloat(count.replace(/\D/g,''));
	if (!counter_on) {
		counter_on = true;
		update_counter();
	}
}

// Sticks the current download value into the DOM, and calls itself every
// update interval to increment the counter.
function update_counter()
{
	var counter = document.getElementById(counter_id);

	if (download_rate) {
		current_count = current_count + download_rate;
	}

	var newcount = document.createElement('span');
	newcount.innerHTML = format_count(current_count);
	counter.appendChild(newcount);
	counter.removeChild(counter.firstChild);

	setTimeout("update_counter()", update_interval);
}

/***************************    Utility Functions   ***************************/

// Calculates the number of downloads per update interval.
function calculate_rate(last_time, last_count)
{
	var time_delta = (last_time - initial_time) / update_interval;
	var count_delta = (last_count - initial_count);

	return count_delta / time_delta;
}

// Round off the current count and stick in commas where appropriate.
function format_count(count)
{
	count = Math.round(count).toString();

	for (var i = count.length - 3; i > 0 ; i -= 3) {
		count = count.slice(0, i) + ',' + count.slice(i, count.length);
	}

	var counter = '';

	for (var i = 0; i < count.length; i++) {
		counter = counter + '<span class="number-' + count.charAt(i).replace(',', 'comma') + '"><b>' + count.charAt(i) + '</b></span>';
	}

	return counter;
}

function start_counter()
{
	var counter = document.getElementById(counter_id);

	if (counter == null) return;

	var imgcache = new Array();
	for (var i = 0; i < 10; i++) {
		imgcache[i] = new Image();
		imgcache[i].src = '/fxcounter/number-' + i + '.png';
	}
	imgcache[10] = new Image();
	imgcache[10].src = '/fxcounter/number-comma.png';
	
	get_count();

}
